/*
 * ClassDef.java
 *
 * Created on April 11, 2003, 3:15 PM
 */
package dbmapper;

import java.util.Vector;
import java.util.HashMap;
import java.util.TreeMap;
import java.util.Set;


import java.io.Serializable;
/**
 * This class maintains info about a single java class
 * including it's name, superclass, classes that extend this
 * class, list of attributes in this class, and a reference
 * to an entity which this class maps to. For typical java classes
 * this reference will be to a database table (TableDef).
 *
 * @author Larry
 */
public class ClassDef implements Serializable, Cloneable {
    
    /** The filter applied during class selection
     */    
    private String filter;
    /** The name of this class
     */    
    private String name;
    /** The mapped flag if we have been mapped
     */    
    private boolean mapped=false;
    /** t/f if we are in the process of being mapped
     */    
    private boolean modinprogress=false;
    
    /** The object we are mapped to
     */    
    private MappedReference obj = null;
 
    /**
     * The filter used to select this class
     * @return The filter
     */    
    public String getFilter(){ return filter; }
    /**
     * Get the name of this class
     * @return The name of this class
     */    
    public String getName(){return name; }
    /**
     * Get the fully qualified name of this class
     * @return The fully qualified name of this class
     */    
    public String getFullName(){ return filter + name; }
    
    /**
     * Is this class mapped to something
     * @return t/f if this class is mapped or not
     */    
    public boolean isMapped(){ return mapped;}
    /**
     * Set the mapped status of this class to true
     */    
    public void setMapped(){ mapped=true; }
    /**
     * Set the mapped status of this class to the given boolean value
     * @param mapflag The flag to set the mapped value to
     */    
    public void setMapped(boolean mapflag){ mapped=mapflag;}
    
    /**
     * Is this class being modified
     * @return t/f if we are being modified or not
     */    
    public boolean isMod(){ return modinprogress;}
    /**
     * Set this class to being modified
     */    
    public void setMod(){ modinprogress=true; }
    /**
     * This the modification in progress flag to the passed value
     * @param modflag t/f flag
     */    
    public void setMod(boolean modflag){ modinprogress=modflag;}
    
    /** The superclass name of this class
     */    
    private String superClass=null;
    /**
     * set the superclass of this class
     * @param superC The superclass of this class
     */    
    public void setSuperClass(String superC){ superClass=superC;}
    /**
     * Get the superclass of this class
     * @return the superclass of this class
     */    
    public String getSuperClass() {return superClass; }
    
    /** A list of interfaces this class implements
     */    
    private java.util.Vector implementList;
    
    /** A list of classes that extend this class
     */    
    private java.util.Vector extList;
    
    /** The attributes of this class
     */    
    private java.util.TreeMap attrs;
    
    /** Creates a new instance of ClassDef
     * @param filt The filter string that is used to select this class form the fully qualified name
     * @param nam The java class name of this class
     */
    public ClassDef(String filt, String nam) {
        filter=filt;
        name=nam;
        attrs=new TreeMap();
        implementList=new Vector();
        extList=new Vector();
    }
    
    /**
     * return a clone of this class
     * @throws CloneNotSupportedException If this class can not be cloned
     * @return The clone of this class
     */    
    public Object clone() throws CloneNotSupportedException{
        return super.clone();
    }
    
    /**
     * Add an attribute for this class
     * @param attrname The name of the attribute
     * @param attrtyp The type of the attribute
     */    
    public void addAttr(String attrname, String attrtyp) {
        if ( attrname == null || attrtyp == null ) return;
        attrs.put(attrname, new AttrDef(attrname, attrtyp));
        return;
    }
    
    /**
     * Get the number of attributes this class contains
     * @return The number of attributes in this class
     */    
    public int numAttrs(){
        return attrs.size();
    }
    
    /**
     * Get a list of attributes in this class
     * @return The list of attributes in this class
     */    
    public String[] getAttrList(){
        Set keyset=attrs.keySet();
        return (String[])keyset.toArray(Utils.emptystringarray);
    }

    /**
     * Get a pretty print definition of the speicified attribute
     * @param attrname The attribute to get a definition of
     * @return The pretty print string defining the attribute
     */    
    public String getPPAttr(String attrname){
        if ( attrname == null ) return null;
        if ( attrs.containsKey(attrname) ){
            return ((AttrDef)attrs.get(attrname)).getPPstr();
        }
        return null;
    }
    
    /**
     * Is the specified attribute mapped
     * @param attrname The name of the attribute to check
     * @return t/f if the attribute is mapped or not
     */    
    public boolean isAttrMapped(String attrname){
        if ( attrname == null ) return false;
        if ( attrs.containsKey(attrname) ){
            return ((AttrDef)attrs.get(attrname)).isMapped();
        }
        return false;
    }
    
    /**
     * Get the AttrDef object for the specified attribute
     * @param attrname The name of the attribute
     * @return The AttrDef object of the specified attribute (null if attribute is not defined)
     */    
    public AttrDef getAttr(String attrname){
        if ( attrname == null ) return null;
        if ( attrs.containsKey(attrname) ){
            return (AttrDef)attrs.get(attrname);
        }
        return null;
    }
        
    /**
     * Get the object that the specified attribute is mapped to
     * @param attrname The attribute name
     * @return The object the attribute is mapped to (null if not defined, or attribute does not exist)
     */    
    public MappedReference getAttrMappedReference(String attrname){
        if ( attrname == null ) return null;
        if ( attrs.containsKey(attrname) ){
            if ( ((AttrDef)attrs.get(attrname)).isMapped() ){
                return ((AttrDef)attrs.get(attrname)).getMappedTo();
            }
        }
        return null;
    }
    
    /**
     * Add a class to the list of implements classes
     * @param implClass The classname to add to the implements list
     */    
    public void addImpl(String implClass){
        implementList.add(implClass);
    }
    /**
     * Get the list of classes that this class implements
     * @return List of classes this class implements
     */    
    public String[] getImplList(){
        return (String[])implementList.toArray(Utils.emptystringarray);
    }
    
    /**
     * Add a class to the list of class that extend this class
     * @param ext A class that extends this class
     */    
    public void addExt(String ext){
        extList.add(ext);
    }
    /**
     * Get the list of classes that extend this class
     * @return The list of classes that extend this class
     */    
    public String[] getExtList(){
        return (String[])extList.toArray(Utils.emptystringarray);
    }
    /**
     * Set the list of classes that extend this class to the passed in list
     * @param list The new list of classes that extend this class
     */    
    public void setExtList(Vector list){
        extList=list;
        return;
    }
        
    
    public void delExt(String ext){
        extList.removeElement(ext);
    }
    
    /**
     * Set the object that this class is mapped to
     * @param o The object this class is mapped to
     */    
    public void setMappedTo(MappedReference o){
        obj=o;
    }
    
    /**
     * Get the object this class is mapped to
     * @return The object this class is mapped to
     */    
    public MappedReference getMappedTo(){
        return obj;
    }
}
