/*
 * ClassFinder.java
 *
 * Created on April 11, 2003, 5:02 PM
 */

package dbmapper;

import java.util.TreeMap;
import java.util.jar.JarFile;
import java.util.jar.JarEntry;
import java.util.Enumeration;
import java.util.Vector;
import java.util.Iterator;
import java.lang.reflect.Field;


/**
 * This class contains the logic to parse a jar file and
 * extract the java classes of interest, pulling out the
 * classes, the attributes of those classes, and building a reverse
 * list of classes that are extended by that class.
 * @author Larry
 */
public class ClassFinder {
    
    /** The jar file to explore
     */    
    String jar=null;
    /** The filter to apply to classes within the jar file
     */    
    String jarfilter=null;
 
    /** A map of classes we pulled from the jar file
     */    
    TreeMap classes=null;
    
    /** A temprary cache used to maintain classes that extend the key field class.
     */    
    TreeMap extendedMap=null;
    
    /** Creates a new instance of ClassFinder given the speicifed configuration
     * object.
     * @param conf The configuration object which contains the jar file location and filter to apply for class loading
     *
     */
    public ClassFinder(Config conf) {
        jar=conf.getJar();
        //System.out.println("ClassFinder:jar=" + jar);
        jarfilter=conf.getJarFilter();
        //System.out.println("ClassFinder:jar=" + jarfilter);
    }
    
    /** Get the list of classes that this ClassFinder has loaded
     * @return A Map of the classes loaded.
     */    
    public TreeMap getClassMap(){ return classes; }

    /**
     * Load the classes from the jar file.
     * @throws ConfigException If a problem occurs opening or reading the jar file.
     *
     */    
    public void load() throws ConfigException {
        try{
            // open the jar file, and get a class loaded
            JarFile jarfile=new JarFile(jar);
            classes=new TreeMap();
            extendedMap = new TreeMap();
            ClassLoader classLoader = ClassLoader.getSystemClassLoader();
            
            // loop over files in the jar file
            for (Enumeration e=jarfile.entries(); e.hasMoreElements(); ){
                JarEntry entry=(JarEntry)e.nextElement();
                String entryName = entry.getName();
                //System.out.println("entry =" + entryName);
                
                // ignore directories
                if ( entryName.endsWith("/") ) continue;
                
                // ignore classes that don't match the filter if speciified
                //JarEntry jentry=jarfile.getJarEntry(entry);
                if ( jarfilter != null && ! jarfilter.equals("") ){
                    //System.out.println("compare {" + entryName + "," + jarfilter + "}");
                    if ( ! entryName.startsWith(jarfilter) )
                        continue;
                }
                
                // make java style classname (.'s instead of /')
                String qualName=entryName.replace('/','.');
                qualName=qualName.substring(0,qualName.indexOf(".class"));
                
                // chopchop
                String shortname=qualName.substring(jarfilter.length());
                //System.out.print("entry to save = " + qualName + "...");
                //System.out.println("shortname = " + shortname);         
                
                // create a new ClassDef, and load the class for introspection
                ClassDef eclass = new ClassDef(jarfilter, qualName);
                Class entryClass = classLoader.loadClass(qualName);
                
                if ( entryClass != null ){
                    //System.out.println(". loaded ok.");
                }else{
                    System.out.println("Unable to load class " + eclass.getName());
                    continue;
                }
                
                //field introspect time;
                Field[] attrs=entryClass.getDeclaredFields();
                for ( int i=0; i<attrs.length; i++){
                    String fldName=attrs[i].getName();
                    Class fldTypClass = attrs[i].getType();
                    String fldTyp=fldTypClass.getName();
                    if ( fldTypClass.isArray() )
                        fldTyp+="[]";
                    //String desc=attrs[i].toString();
                    eclass.addAttr(fldName,fldTyp);
                }
                
                // get implements hierarchy and add to extended map
                Class[] interfaces=entryClass.getInterfaces();
                for ( int i=0; i<interfaces.length; i++ ){
                     eclass.addImpl(interfaces[i].getName());
                }
                
                // get superclass
                Class superC=entryClass.getSuperclass();
                if ( superC != null ){
                    eclass.setSuperClass(superC.getName());
                }
                classes.put(qualName,eclass);
            }
            // now post-process extents list. building a reverse hashmap
            Iterator keyIt=classes.keySet().iterator();
            while ( keyIt.hasNext() ){
                String classname = (String)keyIt.next();
                ClassDef classdef = (ClassDef)classes.get(classname);
                followSupers(classname, classdef.getSuperClass());
            }
            // now process the reverse hashmap and add extents lists back to classes
            keyIt=extendedMap.keySet().iterator();
            while ( keyIt.hasNext() ){
                String classname = (String)keyIt.next();
                ClassDef classdef = (ClassDef)classes.get(classname);
                classdef.setExtList((Vector)extendedMap.get(classname));
            }
        } catch ( Exception e ){
            System.out.println("Exception finding classes " + e);
            e.printStackTrace();
            throw new ConfigException("ClassFinder:load " + e.toString());
        }
    }
    
    // recursive method 
    
    /**
     * recursive method to backtrack through implements
     * clause to find all classes that extend others.
     * @param classnam The classname we are examinng
     * @param superclass The class we implement
     */    
    private void followSupers(String classnam, String superclass){
        if ( superclass == null || superclass.equals("") ) return;
        if ( superclass.equals("java.lang.Object") ) return; //who cares
        if ( ! classes.containsKey(superclass) ){
                System.out.println(" No class def for " + superclass + " during extent examination. I will ignore it.");
                return;
        }
        ClassDef superClassdef = (ClassDef)classes.get(superclass);
        String supersuper = (String)superClassdef.getSuperClass();
        followSupers(classnam, supersuper);
        if ( extendedMap.get(superclass) != null ){
              Vector map=(Vector)extendedMap.get(superclass);
              map.add(classnam);
        }else{
              Vector map=new Vector();
              map.add(classnam);
              extendedMap.put(superclass, map);
        }
        return;
    }

}
