/*
 * ClassTableMap.java
 *
 * Created on April 14, 2003, 6:33 PM
 */

package dbmapper;

import java.util.Vector;
import java.util.HashMap;
import java.util.Enumeration;
import java.io.Serializable;
/** This class defines the mapping between a specified class
 * and a database table during construction of the mapping between
 * the specified class and table. This classes uses clones of the
 * relevant ClassDef and TableDef until mapping is either complete
 * or dropped.<p>
 * The mapping process creates a Vector of mapping pairs matching class-attributes
 * to table-columns. The order in the vector will determine the "id" value
 * in the generated xml mapping file.
 * Once the mapping is complete, this info is stored back into
 * the primary ClassDef, TableDef, etc.
 *
 * @author Larry
 */
public class ClassTableMap implements MappingDescriptor, Serializable {
 
    /** Map type "not applicable" - aka not defined
     */    
    public static final int MAP_TYPE_NA = 0;
    /** mapping type for a primary key
     */    
    public static final int MAP_TYPE_PK = 1;
    /** mapping type for a primitive javatype (ie String, long, int, etc)
     *
     */    
    public static final int MAP_TYPE_PRIM = 2;
    /** mapping type for an object reference
     */    
    public static final int MAP_TYPE_OBJREF = 3;
    /** mapping type for a dodo attribute (ignore attributes)
     */    
    public static final int MAP_TYPE_DATTR = 4;
    /** mapping type for a dodo column (ignored column)
     */    
    public static final int MAP_TYPE_DCOL = 5;
    /** mapping type for a collection (array of objects - 1:many)
     */    
    public static final int MAP_TYPE_COLLECT = 6;
    /** mapping type for a collection (many:many reference)
     */    
    public static final int MAP_TYPE_MANY = 7;
    
    /** The sourceProcessor this class uses to get info about classes and tables
     */    
    private SourceProcessor myProc=null;
    /** The tablename this class is mapping
     */    
    private String tablename=null;
    /** The classname this class is mapping
     */    
    private String classname=null;
    
    /** The "local" copy of the ClassDef for the class this class is mapping
     */    
    private ClassDef myClassDef=null;
    /** The "local" copy of the TableDef for the table this class is mapping
     */    
    private TableDef myTableDef=null;
    
    //caches
    /** The list of attributes for the class we are mapping
     */    
    private String[] attrlist;
    /** The list of columns for the table we are mapping
     */    
    private String[] collist;
    
    /** A map of attribute pretty-print-names to actual attribute name
     */    
    private HashMap attrlookup;
    /** A map of column pretty-print-names to actual column name
     */    
    private HashMap collookup;
    
    /** A list of many-many association mappings that have been established
     */    
    private Vector manymaps=new Vector();;
    
    /** The order of attribute-column mappings
     */    
    private Vector mappingOrder=new Vector();
    
    /** A cached Vector we use throughout several methods (reduce instan time)
     */    
    private Vector results=new Vector(); //cache a Vector object for multiple use & speed
    
    /** Creates a new instance of ClassTableMap */
    public ClassTableMap(){
    }
    
    /**
     * Creates a new instance of ClassTableMap with the prime
     * elements specified (classname, tablename, and processor)
     * @param proc The SourceProcessor to use for getting class/table info
     * @param classnam The classname to map
     * @param tablenam The tablename to map
     */    
    public ClassTableMap(SourceProcessor proc, String classnam, String tablenam) {
        classname=classnam;
        tablename=tablenam;
        myProc = proc;
        this.load();
    }
    
    // get relevant data for class-table mapping
    /**
     * Get the class/table data from the processor to begin
     * the mapping process.
     */    
    public void load(){
        // get class and table dups for local mod
        myClassDef=myProc.getClassDefClone(classname);
        myTableDef=myProc.getTableDefClone(tablename);
      
        attrlist = myClassDef.getAttrList();
        collist=myTableDef.getColList();
        
        attrlookup=new HashMap();
        for ( int i=0; i<attrlist.length; i++ ){
            attrlookup.put(myClassDef.getPPAttr(attrlist[i]), attrlist[i]);
        }
        collookup=new HashMap();
        for ( int i=0; i<collist.length; i++ ){
            collookup.put(myTableDef.getPPCol(collist[i]), collist[i]);
        }
    }
    
    /** Get the classname this class is mapping
     * @return The classname this class is mapping
     */    
    public String getClassname() {return classname; }
    
    /** Get the tablename this class is mapping
     * @return The tablename
     */    
    public String getTablename() {return tablename; }
    
    /** Set the classname this class is mapping
     * @param name The classname to map
     */    
    void setClassname(String name) {classname=name; }
    
    /** Set the tablename this class is to map
     * @param name The tablename to map
     */    
    void setTablename(String name) {tablename=name; }
    
    /** get the ClassDef object for the class this class is mapping
     * @return The ClassDef object of the class being mapped
     */    
    public ClassDef getClassDef(){ return myClassDef;}
    /** Get the TableDef object for the table this class is mapping
     * @return The TableDef of the table being mapped
     */    
    public TableDef getTableDef(){ return myTableDef;}

    /** Get a list of attributes of the class we are mapping
     * that have not been mapped yet.
     * @return List of unmapped Attribute names
     */    
    public Object[] getUnmappedAttrs(){
        results.clear();
        for ( int i=0; i<attrlist.length; i++ ){
            if ( myClassDef.isAttrMapped(attrlist[i]) )continue;
            results.add(myClassDef.getPPAttr(attrlist[i]));
        }
        return (String[])results.toArray(Utils.emptystringarray);
    }
    
    /** Get a list of columns for the table we are mapping
     * that have not been mapped yet.
     * @return The list of unmapped column names.
     */    
    public Object[] getUnmappedCols(){
        results.clear();
        for ( int i=0; i<collist.length; i++ ){
            if ( myTableDef.isColMapped(collist[i]) )continue;
            results.add(myTableDef.getPPCol(collist[i]));
        }
        return (String[])results.toArray(Utils.emptystringarray);
    }
    
    /** Get the list of classes that extend the class that is being
     * mapped.
     * @return List of extending classes
     */    
    public Object[] getExtList(){
        return myProc.getClassExtList(classname);
    }
    
    
    /** Move the specified mapping pair up one slot in the order
     * of mappings.
     * @param index The index of the desired mapping pair to move
     */    
    public void moveMappedUp(int index){
        if ( index < 1 || index > mappingOrder.size() ) return;
        Object above = mappingOrder.get(index-1);
        mappingOrder.set(index-1, mappingOrder.get(index));
        mappingOrder.set(index, above );
        return;
    }
    
    /** Move the specified mapping pair down one slot in the order
     * of mappings.
     * @param index The index of the desired mapping pair to move
     */    
    public void moveMappedDown(int index){
        if ( index == mappingOrder.size() ) return;
        Object below = mappingOrder.get(index+1);
        mappingOrder.set(index+1, mappingOrder.get(index));
        mappingOrder.set(index, below );
        return;
    }
    
    // get mapped description for all mapped attributes.
    /** Get a pretty print list of mappings that have been defined
     * @return A list of mappings defined
     */    
    public String[] getMappedDescrition(){
        String[] results=new String[mappingOrder.size()];
        for ( int i=0; i<mappingOrder.size(); i++ ){
            MappedReference mapref=(MappedReference)mappingOrder.elementAt(i);
            results[i]="#"+i+": " + this.getMappedDescription(mapref);
        }
        return results;
    }

    /** Get a pretty print description of the specified reference
     *
     * @param ref The reference to describe
     * @return A pretty print string describing the reference
     */    
    public String getMappedDescription(MappedReference ref){
        switch (ref.getMapType() ){
           case  MAP_TYPE_NA:   return ref.getName() + " n/a?";
           case  MAP_TYPE_PK:   return ref.getPPstr() + " -PK-> " +
                                ((MappedReference)ref.getMappedTo()).getPPstr();
           case  MAP_TYPE_PRIM: return ref.getPPstr() + " -Primitive-> " +
                                ((MappedReference)ref.getMappedTo()).getPPstr();
            case  MAP_TYPE_OBJREF: return ref.getPPstr() + " -ObjectRef-> " +
                                ((MappedReference)ref.getMappedTo()).getPPstr();
            case  MAP_TYPE_DATTR: return ref.getPPstr() + " -Ignore-| ";
            case MAP_TYPE_DCOL:   return "           |-Ignore-" + ref.getPPstr();
            case  MAP_TYPE_COLLECT: return ref.getPPstr() + " -Collection of { " +
                                ((MappedReference)ref.getMappedTo()).getPPstr(); 
            case  MAP_TYPE_MANY: return ref.getPPstr() + " -Many2Many- " + 
                                ((MappedReference)ref.getMappedTo()).getPPstr(); 
        }
        return null;
    }
    
    /** Get the mapping source that would be generated for this
     * mapping
     * @return The xml mapping code
     */    
    public String getMapSource(){
        return Utils.classMapXmlCode(this);
    }
    
    /** Make a primary key reference between the attribute and tablename
     * specified
     * @param attrname The attribute name
     * @param colname The column name
     */    
    public void makePKref(String attrname, String colname){
        AttrDef attr=myClassDef.getAttr((String)attrlookup.get(attrname));
        ColDef col=myTableDef.getCol((String)collookup.get(colname));
        attr.setMappedTo((MappedReference)col);
        col.setMappedTo((MappedReference)attr);
        attr.setMapType(MAP_TYPE_PK);
        col.setMapType(MAP_TYPE_PK);
        attr.setMapProp(MappingProperties.newMappingProperty(MAP_TYPE_PK));
        mappingOrder.add(attr);
    }
    
    /** Make a primitive type reference between the attribute and tablename
     * specified
     * @param attrname The attribute name
     * @param colname The column name
     */    
    public void makePrimref(String attrname, String colname){
        AttrDef attr=myClassDef.getAttr((String)attrlookup.get(attrname));
        ColDef col=myTableDef.getCol((String)collookup.get(colname));
        attr.setMappedTo((MappedReference)col);
        col.setMappedTo((MappedReference)attr);
        attr.setMapType(MAP_TYPE_PRIM);
        col.setMapType(MAP_TYPE_PRIM);
        attr.setMapProp(MappingProperties.newMappingProperty(MAP_TYPE_PRIM));
        mappingOrder.add(attr);
    }
    
    /** Make an object reference between the attribute and tablename
     * specified
     * @param attrname The attribute name
     * @param colname The column name
     */    
    public void makeObjref(String attrname, String colname){
        AttrDef attr=myClassDef.getAttr((String)attrlookup.get(attrname));
        ColDef col=myTableDef.getCol((String)collookup.get(colname));
        attr.setMappedTo((MappedReference)col);
        col.setMappedTo((MappedReference)attr);
        attr.setMapType(MAP_TYPE_OBJREF);
        col.setMapType(MAP_TYPE_OBJREF);
        attr.setMapProp(MappingProperties.newMappingProperty(MAP_TYPE_OBJREF));
        mappingOrder.add(attr);
    }
    
    /** Make a collection reference between the attribute and the database
     * column map specified. The collmap specifies a different table and
     * column pair than the ones this class is working with (the nature of
     * collections :)
     *
     * @param attrname The name of the attribute
     * @param collmap The collmap to map to
     */    
    public void makeCollref(String attrname, CollMap collmap){
        //one-way link for collections
        AttrDef attr=myClassDef.getAttr((String)attrlookup.get(attrname));
        ClassDef fkclass=myProc.getClassDef(collmap.getClassname());
        AttrDef fkattr=fkclass.getAttr(collmap.getFktome());
        attr.setMappedTo((MappedReference)fkattr);
        attr.setMapType(MAP_TYPE_COLLECT);
        attr.setMapProp(MappingProperties.newMappingProperty(MAP_TYPE_COLLECT));
        mappingOrder.add(attr);
    }
    
    
    /** Make a collection reference between the attribute and the
     * many map specified. The many specifies a different table and
     * two columns pair than the ones this class is working with
     * (the nature of many to many collections :)
     * @param attrname The attribute name
     * @param manyMap The manyMap many:many definition
     */    
    public void makeManyref(String attrname, ManyMap manyMap){
        AttrDef attr=myClassDef.getAttr((String)attrlookup.get(attrname));
        attr.setMappedTo((MappedReference)manyMap);
        manyMap.setMappedTo((MappedReference)attr);
        attr.setMapType(MAP_TYPE_MANY);
        attr.setMapProp(MappingProperties.newMappingProperty(MAP_TYPE_MANY));
        mappingOrder.add(attr);
        manymaps.add(manyMap);
    }
    
    /** Make an attribute a dud attribute (ie not used)
     * @param attrname The attribute name
     */    
    public void makeDudClassref(String attrname){
        AttrDef attr=myClassDef.getAttr((String)attrlookup.get(attrname));
        attr.setMappedTo((MappedReference)attr);
        attr.setMapType(MAP_TYPE_DATTR);
        mappingOrder.add(attr);
    }
    
    /** Make a column a dud column (ie not used)
     * @param colname The column name
     */    
    public void makeDudDbcolref(String colname){
        ColDef col=myTableDef.getCol((String)collookup.get(colname));
        col.setMappedTo((MappedReference)col);
        col.setMapType(MAP_TYPE_DCOL);
    }
    
    /** Get an enumeration of the mappings defined in the order
     * they have been set to.
     * @return An enumeration of the mappings
     */    
    public Enumeration getMapping(){
        return mappingOrder.elements();
    }
     
    /** Remove the mapping pair at the specified index.
     * @param index The mapping pair to remove
     */    
    public void unMap(int index){
        AttrDef attr = (AttrDef)mappingOrder.get(index);
        if ( attr.getMapType() == MAP_TYPE_MANY ){
            manymaps.remove(attr.getMappedTo());
        }else if ( attr.getMapType() == MAP_TYPE_DATTR ){
            //
        }else{
            MappedReference mr=attr.getMappedTo();
            mr.setMappedTo(null);
            mr.setMapType(MAP_TYPE_NA);
        }
        attr.setMappedTo(null);
        attr.setMapType(MAP_TYPE_NA);
        mappingOrder.remove(index);
        return;
    }
    
    /** get tha Mapping properties for the mapping pair at the index specified.
     * @param index The mapping pair of interest
     * @return The properties of the mapping pair.
     */    
    public MappingProperties getMappedProperties(int index){
        if ( index < 0 || index > mappingOrder.size() ) return null;
        AttrDef attr = (AttrDef)mappingOrder.get(index);
        return attr.getMapProp();
    }
    
    public void addExt(String classnam){
        myClassDef.addExt(classnam);
    }
    
    public void delExt(String classnam){
        myClassDef.delExt(classnam);
    }
}
