/*
 * DbAccess.java
 *
 * Created on April 11, 2003, 7:33 AM
 */

package dbmapper;

import java.io.PrintWriter;
import java.util.TreeMap;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.SQLWarning;


/** This class handles database connectivity. It will open a connection to the
 * configured database and fetch the list of tables and columns. The SQL code
 * in this class was established on an Oracle database.<p>
 * Typical use:<p>
 *  DbAccess dba=new DbAccess(configObject)<p>
 *  dba.load();<p>
 *  TreeMap tableMap = dba.getTableMap();<p>
 *  dba.close();<p>
 * @author Larry
 */
public class DbAccess {
    
    /** The jdbc driver class
     */    
    String jdbcdriver=null;
    /** The db url
     */    
    String dburl=null;
    /** The db userid
     */    
    String dbuserid=null;
    /** The db password
     */    
    String dbpass=null;
    /** The schema (unused?)
     */    
    String dbschema=null;
    
    /** The jdbc db connection
     */    
    Connection con=null;
    
    /** The db table list
     */    
    TreeMap tables=null;
    
    /** Creates a new instance of DbAccess
     * @param conf The configuration settings object
     */
    public DbAccess(Config conf) {
        jdbcdriver=conf.getJdbcDriver();
        dburl=conf.getUrl();
        dbuserid=conf.getUserid();
        dbpass=conf.getPassword();
        dbschema=conf.getSchema();
    }
    
    /** Get the list of tables. A TreeMap object is returned with the key being the
     * table name and a TableDef object as the value in the Map.
     * @return The TreeMap list of tables.
     * @see dbmapper.TableDef
     */    
    public TreeMap getTableMap(){ return tables; }
    
    /** Fetch the tabel and column info from the db.
     * @throws ConfigException If a problem occurs getting db info
     */    
    public void load() throws ConfigException {
        try{
	    // Load the driver
	    Class.forName (jdbcdriver);
	    DriverManager.setLogWriter(new PrintWriter(System.out));

	    // Attempt to connect to driver.
	    con = DriverManager.getConnection (
			dburl, dbuserid, dbpass);

	    // Check for, and display and warnings generated
	    // by the connect.
	    checkForWarning (con.getWarnings ());

	    // get table list
	    ResultSet rs = execute_statement ("select Table_Name, Column_Name, Data_Type " +
                           "from USER_TAB_COLUMNS");
            tables=new TreeMap();
            String current_table="";
            TableDef tableDef=null;
            String tabname=null;
            while ( rs.next() ){
                tabname=rs.getString("table_name");
                String colname=rs.getString("Column_Name");
                String coltyp=rs.getString("Data_Type");
                //System.out.println("T:"+tabname+" C:"+colname+" t:" + coltyp);
                if ( ! tabname.equals(current_table) ){
                    if ( tableDef != null ){
                        tables.put(current_table, tableDef);
                    }
                    tableDef=new TableDef(tabname);
                    current_table=tabname;
                    //System.out.println(" New Table:" + tabname);
                }
                tableDef.addCol(colname,coltyp);
            }
            if ( tableDef != null ){
                tables.put(tabname, tableDef);
            }
            close();
	} catch (SQLException ex) {
            System.out.println ("\n*** SQLException caught ***\n");
	    while (ex != null) {
		System.out.println ("SQLState: " +
				ex.getSQLState ());
		System.out.println ("Message:  " + ex.getMessage ());
		System.out.println ("Vendor:   " +
				ex.getErrorCode ());
		ex = ex.getNextException ();
		System.out.println ("");
		}
            throw new ConfigException("SQlException in db load.");
	} catch (java.lang.Exception ex) {
		// Got some other type of exception.  Dump it.
		ex.printStackTrace ();
                throw new ConfigException(ex.toString());
	}
    }
    
    /** Probably not needed as a seperate method, but I thought there may need to be
     * other sql calls
     * @param statement The SQL statement to execute
     * @return The jdbc result set
     */    
    public ResultSet execute_statement(String statement){
         try {
	    // Create a Statement object so we can submit
	    // SQL statements to the driver
	    Statement stmt = con.createStatement ();

	    // Submit a query, creating a ResultSet object
	    ResultSet rs = stmt.executeQuery (statement);

            //stmt.close();
            return rs;
	} catch (SQLException ex) {
            System.out.println ("\n*** SQLException caught ***\n");
	    while (ex != null) {
		System.out.println ("SQLState: " +
				ex.getSQLState ());
		System.out.println ("Message:  " + ex.getMessage ());
		System.out.println ("Vendor:   " +
				ex.getErrorCode ());
		ex = ex.getNextException ();
		System.out.println ("");
	    }
            return null;
	} catch (java.lang.Exception ex) {
		// Got some other type of exception.  Dump it.
		ex.printStackTrace ();
                return null;
	}
    }

    //-------------------------------------------------------------------
    // checkForWarning
    // Checks for and displays warnings.  Returns true if a warning
    // existed
    //-------------------------------------------------------------------
 
    /** Check the results of the jdbc execute for errors
     * @param warn jdbc warning
     * @throws SQLException for SQL Exception
     * @return t/f if a warning occurred
     */    
    private static boolean checkForWarning (SQLWarning warn) 	
		throws SQLException  {
	boolean rc = false;
	// If a SQLWarning object was given, display the
	// warning messages.  Note that there could be
	// multiple warnings chained together
	if (warn != null) {
		System.out.println ("\n *** Warning ***\n");
		rc = true;
		while (warn != null) {
			System.out.println ("SQLState: " +
				warn.getSQLState ());
			System.out.println ("Message:  " +
				warn.getMessage ());
			System.out.println ("Vendor:   " +
				warn.getErrorCode ());
			System.out.println ("");
			warn = warn.getNextWarning ();
		}
	}
	return rc;
    }

    /** Close the db connection.
     */    
    public void close(){
         try {
            //close connection
            con.close();
	} catch (SQLException ex) {
            System.out.println ("\n*** SQLException caught ***\n");
	    while (ex != null) {
		System.out.println ("SQLState: " +
				ex.getSQLState ());
		System.out.println ("Message:  " + ex.getMessage ());
		System.out.println ("Vendor:   " +
				ex.getErrorCode ());
		ex = ex.getNextException ();
		System.out.println ("");
		}
	} catch (java.lang.Exception ex) {
		// Got some other type of exception.  Dump it.
		ex.printStackTrace ();
	}
    }

    
    
}
